# frozen_string_literal: true

module Ros
  class ApplicationController < ::ApplicationController
    include JSONAPI::ActsAsResourceController
    rescue_from Pundit::NotAuthorizedError, with: :user_not_authorized
    include OpenApi::DSL

    before_action :authenticate_it!
    before_action :set_raven_context, if: -> { Settings.credentials.sentry_dsn }

    def authenticate_it!
      return unless @current_user = request.env['warden'].authenticate!(:api_token)
      # set_jwt if request.env['HTTP_AUTHORIZATION'].starts_with?('Basic')
      response.set_header('AUTHORIZATION', "Bearer #{jwt}") if request.env['HTTP_AUTHORIZATION'].starts_with?('Basic')

      # render(status: :unauthorized, json: { errors: [{
      #   status: 401, code: 'unauthorized', title: 'Unauthorized'
      # }]})
      # throw(:abort) unless @current_user
    end

    # def set_jwt; response.set_header('AUTHORIZATION', "Bearer #{jwt}") end

    def jwt; Jwt.encode(current_user.jwt_payload) end

    def current_user;  @current_user end

    # Methods for Pundit
    def context; { user: current_user } end
    def user_not_authorized; head :forbidden end

    def set_raven_context
      # Raven.user_context(id: session[:current_user_id]) # or anything else in session
      Raven.extra_context(params: params.to_unsafe_h, url: request.url, tenant: Apartment::Tenant.current)
    end

    # Documentation
    api_dry [:index, :show] do
      query :page, Integer
    end
  end
end
