# frozen_string_literal: true

module Ros
  # ALL models inherit from this class
  class ApplicationRecord < ::ApplicationRecord
    self.abstract_class = true
    include ApiBelongsTo

    # urn:partition:service:region:account_id:resource_type
    # def self.to_urn; "#{urn_base}:#{current_tenant.try(:account_id)}:#{name.underscore}" end
    def self.to_urn; "#{urn_base}:#{account_id}:#{name.underscore}" end

    def self.account_id
      Apartment::Tenant.current.eql?('public') ? '' : Apartment::Tenant.current.remove('_')
    end

    def self.current_tenant; Tenant.find_by(schema_name: Apartment::Tenant.current) end

    # Universal Resource Name (URNs) and Service Namespaces
    # urn:partition:service:region
    def self.urn_base; "urn:#{Settings.service.partition_name}:#{Settings.service.name}:#{Settings.service.region}" end

    def self.find_by_urn(value); find_by(urn_id => value) end

    # urn:partition:service:region:account_id:resource_type/id
    def to_urn; "#{self.class.to_urn}/#{send(self.class.urn_id)}" end

    def current_tenant; self.class.current_tenant end

    # NOTE: Override in model to provide a custom id
    def self.urn_id; :id end
  end
end
