# frozen_string_literal: true

module Ros
  class ApplicationPolicy
    attr_reader :user, :record

    def self.actions
      descendants.reject{ |d| d.name.eql? 'ApplicationPolicy' }.each_with_object([]) do |policy, ary|
        ary.concat(policy.accepted_actions.values.flatten)
      end.uniq
    end

    def self.policies
      descendants.reject{ |d| d.name.eql? 'ApplicationPolicy' }.each_with_object([]) do |policy, ary|
        ary.concat(policy.accepted_policies.values.flatten)
      end.uniq
    end
  
    def initialize(user, record)
      @user = user
      @record = record
    end
  
    # UserPolicy.new({ policies: ['IamFullAccess'] }, nil).index?
    def index?
      standard_check?
    end
  
    def show?
      standard_check?
    end
  
    def create?
      standard_check?
    end
  
    def new?
      create?
    end
  
    def update?
      standard_check?
    end
  
    def edit?
      update?
    end
  
    def destroy?
      standard_check?
    end

    class Scope
      attr_reader :user, :scope
  
      def initialize(user, scope)
        @user = user
        @scope = scope
      end
  
      def resolve
        scope.all
      end
    end
  
    # def self.policies
    #   {
    #     "#{policy_name}FullAccess": {
    #       Effect: 'Allow',
    #       Action: "#{policy_name}:*",
    #       Resource: '*'
    #     },
    #     "#{policy_name}ReadOnlyAccess": {
    #       Effect: 'Allow',
    #       Action: ["#{policy_name}:Get*", "#{policy_name}:List*"],
    #       Resource: '*'
    #     }
    #   }
    # end
    #
    def service; Settings.service.name.eql?('iam') ? :local : :remote end

    def standard_check?
      action = caller_locations(1,1)[0].label.to_sym
# Just like apartment, this will need code for if in IAM or in remote
      if service.eql?(:local)
        (user.policies.pluck(:name) & accepted_policies(action)).any? || (user.actions.pluck(:name) & accepted_actions(action)).any?
      else
        (user.policies & accepted_policies(action)).any? || (user.actions.pluck(:name) & accepted_actions(action)).any?
      end
    end

    def accepted_policies(action); self.class.accepted_policies[action] || [] end
    def accepted_actions(action); self.class.accepted_actions[action] || [] end

    def self.accepted_policies
      {
        index?: [
          "AdministratorAccess",
          "#{policy_name}FullAccess",
          "#{policy_name}ReadOnlyAccess",
        ],
        show?: [
          "AdministratorAccess",
          "#{policy_name}ReadOnlyAccess",
        ],
        create?: [
          "AdministratorAccess",
          "#{policy_name}FullAccess",
        ],
        update?: [
          "AdministratorAccess",
          "#{policy_name}FullAccess",
        ],
        destroy?: [
          "AdministratorAccess",
          "#{policy_name}FullAccess",
        ]
      }
    end

    def self.accepted_actions
      {
        index?: [
          "#{policy_name}List#{model_name.pluralize}"
        ],
        create?: [
          "#{policy_name}Create#{model_name}"
        ]
      }
    end

    def self.policy_name; Settings.service.policy_name end

    def self.model_name
      "#{name.gsub('Policy', '')}"
    end
  end
end
